from spinnman.messages.scp import SCPRequestHeader
from spinnman.messages.scp.abstract_messages import AbstractSCPRequest
from spinnman.messages.scp.enums import SCPCommand
from spinnman.messages.sdp import SDPFlag, SDPHeader
from .iptag_get_info_response import IPTagGetInfoResponse

_IPTAG_INFO = 4
_IPTAG_MAX = 255


class IPTagGetInfo(AbstractSCPRequest):
    """ An SCP Request information about IP tags
    """

    def __init__(self, x, y):
        """
        :param x: The x-coordinate of a chip, between 0 and 255
        :type x: int
        :param y: The y-coordinate of a chip, between 0 and 255
        :type y: int
        """
        super(IPTagGetInfo, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=0, destination_chip_x=x,
                destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_IPTAG),
            argument_1=(_IPTAG_INFO << 16),
            argument_2=_IPTAG_MAX)

    def get_scp_response(self):
        return IPTagGetInfoResponse()
